package org.bitbucket.bradleysmithllc.etlunit.feature;

import com.github.fge.jackson.JsonLoader;
import com.github.fge.jsonschema.exceptions.ProcessingException;
import com.github.fge.jsonschema.main.JsonSchema;
import com.github.fge.jsonschema.main.JsonSchemaFactory;

import java.io.IOException;
import java.util.Arrays;
import java.util.List;

public class ResourceFeatureAnnotation implements FeatureAnnotation
{
	private final String name;
	private final String description;
	private final ResourceFeatureMetaInfo resourceFeatureMetaInfo;
	private final propertyType pType;

	public ResourceFeatureAnnotation(ResourceFeatureMetaInfo resourceFeatureMetaInfo, String key, String description, propertyType prop)
	{
		name = key;
		this.description = description;
		this.resourceFeatureMetaInfo = resourceFeatureMetaInfo;
		pType = prop;
	}

	@Override
	public String getName()
	{
		return name;
	}

	@Override
	public String getDescription()
	{
		return description;
	}

	@Override
	public List<JsonSchema> getValidator()
	{
		String schema = resourceFeatureMetaInfo.getResource(name + ".validator.jsonSchema");

		if (schema != null)
		{
			try
			{
				JsonSchemaFactory fact = JsonSchemaFactory.byDefault();

				JsonSchema res = fact.getJsonSchema(JsonLoader.fromString(schema));

				return Arrays.asList(res);
			}
			catch (IOException e) {
				throw new RuntimeException("Invalid schema for annotation signature [" + resourceFeatureMetaInfo.getReportableFeatureName()
						+ "].[" + name + "]: " + e.toString());
			} catch (ProcessingException e) {
				throw new RuntimeException("Invalid schema for annotation signature [" + resourceFeatureMetaInfo.getReportableFeatureName()
						+ "].[" + name + "]: " + e.toString());
			}
		}

		return null;
	}

	@Override
	public propertyType getPropertyType()
	{
		return pType;
	}
}